/* 
 * Blinky - a GTK network usage monitor
 *
 * Copyright (C) 2002, Benjamin Miller <bmiller@users.sf.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


/* usage-proc.c - get device usage using the Linux /proc/net/dev interface */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include "blinky.h"


static char *parse_line (char *);


/* Linux version of get_usage_all -- uses /proc/net/dev to get tx and
   rx counts for all the IFAs */
void
get_usage_all (void)
{
    int fd;
    static char buf[2048];
    char *pos;

    if ((fd = open("/proc/net/dev", O_RDONLY)) == -1) {
	perror ("opening /proc/net/dev");
	exit(1);
    }

    memset (buf, 0, sizeof(buf));

    if (read (fd, buf, sizeof(buf)) == -1) {
	perror ("reading /proc/net/dev");
	exit(1);
    }

    close (fd);

    pos = buf;

    while (*(pos++) != '\n') ;
    while (*(pos++) != '\n') ;

    /* parse each line */
    while (pos != NULL)
	pos = parse_line (pos);

    

    return;
}


/* Utility function for Linux get_usage_all() - parses one line out of 
   /proc/net/dev, and fills in the appropriate IFA if it matches any */
static char *
parse_line (char *pos)
{
    int i;
    unsigned long tx, rx;
    char name[16];
    IFA *ifa = NULL;
    GSList *l;

    /* get name */
    while (*pos == ' ') pos++;
    sscanf (pos, "%s", name);
    name[strcspn(name,":")] = '\0';

    /* look for an IFA with this name */
    l = ifs;
    while (l != NULL) {
	IFA *f = (IFA *)l->data;
	if (!strcmp (name, f->name)) {
	    ifa = f;
	    break;
	}
	l = l->next;
    }

    /* quit if no interface was found for this line */
    if (ifa == NULL) {
	while (*pos != '\n' && *pos != '\0') pos++;
	if (*pos == '\0')
	    return NULL;
	pos++;
	return pos;
    }

    /* found the interface -- ifa */
    ifa->found = TRUE;
    /* skip over "name:" */
    while (*pos != ':' && *pos != '\0') pos++;
    if (*pos) pos++;
    /* get rx and tx counts */
    sscanf (pos, "%lu %*d %*d %*d %*d %*d %*d %*d %lu", &rx, &tx);

    /* skip to beginning of tx count */
//    sscanf (pos, "%*d %*d %*d %*d %*d %*d %*d %*d %lu", &tx);

    /* skip to end of line */
    while (*pos != '\n' && *pos != '\0') pos++;
    if (*pos == '\0')
	return NULL;

    /* value of -1 means reset to the newly read value */
    if (ifa->tx_cur == -1)
	ifa->tx_cur = tx;
    if (ifa->rx_cur == -1)
	ifa->rx_cur = rx;

    /* is the interface transmitting? */
    if (tx > ifa->tx_cur)
	ifa->txing = TRUE;
    else
	ifa->txing = FALSE;
    /* is it receiving? */
    if (rx > ifa->rx_cur)
	ifa->rxing = TRUE;
    else
	ifa->rxing = FALSE;
    /* is it up? yes. */
    if (ifa->up == FALSE)
	ifa->up = TRUE;

    /* set to newly read values */
    ifa->tx_cur = tx;
    ifa->rx_cur = rx;

    pos++;
    return pos;

}


/*
 * Local variables:
 * c-basic-offset: 4
 * End:
 */
